@@ -304,8 +304,7 @@ init_cpu_info(struct dr_info *dr_info)
 	for (drc = drc_list; drc; drc = drc->next) {
 		cpu = alloc_dr_node(drc, CPU_DEV, NULL);
 		if (cpu == NULL) {
-			say(ERROR, "Could not allocate CPU node structure: "
-			    "%s\n", strerror(errno));
+			say(ERROR, "Could not allocate CPU node structure\n");
 			free_node(cpu_list);
 			return -1;
 		}
@@ -415,7 +414,7 @@ cache_get_dependent_cache(struct cache_info *cache, struct dr_info *dr_info)
 		if (cache->removed)
 			continue;
 
-		if (cache->phandle == cache->l2cache) {
+		if (c->phandle == cache->l2cache) {
 			say(EXTRA_DEBUG, "found cache %s for cache %s\n",
 			    c->name, cache->name);
 			return c;
@@ -502,6 +501,7 @@ free_cache_info(struct cache_info *cache_list)
 
 	while (tmp != NULL) {
 		cache_list = cache_list->next;
+		free((char *)tmp->path);
 		free(tmp);
 		tmp = cache_list;
 	}
@@ -554,7 +554,7 @@ init_cache_info(struct dr_info *dr_info)
 			}
 
 			snprintf(cache->name, DR_BUF_SZ, "%s", ent->d_name);
-			snprintf(cache->path, DR_BUF_SZ, "%s", path);
+			cache->path = strdup(path);
 
 			cache->removed = 0;
 			cache->next = cache_list;
@@ -657,7 +657,6 @@ acquire_cpu(struct dr_node *cpu, struct dr_info *dr_info)
 	return 0;
 }
 
-#ifdef NOT_YET
 int do_cpu_kernel_dlpar(struct dr_node *cpu, int action)
 {
 	char cmdbuf[256];
@@ -681,7 +680,6 @@ int do_cpu_kernel_dlpar(struct dr_node *cpu, int action)
 
 	return do_kernel_dlpar(cmdbuf, offset);
 }
-#endif
 
 int
 probe_cpu(struct dr_node *cpu, struct dr_info *dr_info)
@@ -691,11 +689,9 @@ probe_cpu(struct dr_node *cpu, struct dr_info *dr_info)
 	int write_len;
 	int rc = 0;
 
-#ifdef NOT_YET
 	if (kernel_dlpar_exists()) {
 		rc = do_cpu_kernel_dlpar(cpu, ADD);
 	} else {
-#endif
 		probe_file = open(CPU_PROBE_FILE, O_WRONLY);
 		if (probe_file <= 0) {
 			/* Attempt to add cpu from user-space, this may be
@@ -731,9 +727,7 @@ probe_cpu(struct dr_node *cpu, struct dr_info *dr_info)
 
 			close(probe_file);
 		}
-#ifdef NOT_YET
 	}
-#endif
 
 	if (!rc) {
 		update_cpu_node(cpu, NULL, dr_info);
@@ -797,10 +791,8 @@ release_cpu(struct dr_node *cpu, struct dr_info *dr_info)
 	int release_file;
 	int rc;
 
-#ifdef NOT_YET
 	if (kernel_dlpar_exists())
 		return do_cpu_kernel_dlpar(cpu, REMOVE);
-#endif
 
 	release_file = open(CPU_RELEASE_FILE, O_WRONLY);
 	if (release_file > 0) {
