@@ -174,14 +174,17 @@ struct dr_connector *search_drc_list(struct dr_connector *,
 				     struct dr_connector *, int, void *);
 
 int get_my_drc_index(char *, uint32_t *);
+int get_my_partner_drc_index(struct dr_node *, uint32_t *);
 int drc_name_to_index(const char *, struct dr_connector *);
 char * drc_index_to_name(uint32_t, struct dr_connector *);
 int get_drc_by_name(char *, struct dr_connector *, char *, char *);
+int get_drc_by_index(uint32_t, struct dr_connector *, char *, char *);
 
 int get_min_common_depth(void);
 int get_assoc_arrays(const char *dir, struct assoc_arrays *aa,
 		     int min_common_depth);
 int of_associativity_to_node(const char *dir, int min_common_depth);
+int init_node(struct dr_node *);
 
 static inline int aa_index_to_node(struct assoc_arrays *aa, uint32_t aa_index)
 {
