@@ -36,6 +36,7 @@
 #include "lparstat.h"
 #include "pseries_platform.h"
 #include "cpu_info_helpers.h"
+#include <time.h>
 
 #define LPARCFG_FILE	"/proc/ppc64/lparcfg"
 #define SE_NOT_FOUND	"???"
@@ -255,14 +256,17 @@ long long get_delta_value(char *se_name)
 
 void get_time()
 {
-	struct timeval t;
 	struct sysentry *se;
+	struct timespec ts;
+	int err;
 
-	gettimeofday(&t, 0);
+	err = clock_gettime(CLOCK_BOOTTIME, &ts);
+	if (err)
+		return;
 
 	se = get_sysentry("time");
 	sprintf(se->value, "%lld",
-		(long long)t.tv_sec * 1000000LL + (long long)t.tv_usec);
+		(long long)ts.tv_sec);
 }
 
 int get_time_base()
@@ -304,7 +308,6 @@ double get_scaled_tb(void)
 	online_cores = atoi(se->value);
 
 	elapsed = get_delta_value("time");
-	elapsed = elapsed / 1000000.0;
 
 	se = get_sysentry("timebase");
 	timebase = atoi(se->value);
@@ -312,31 +315,6 @@ double get_scaled_tb(void)
 	return (timebase * elapsed) * online_cores;
 }
 
-void get_sys_uptime(struct sysentry *unused_se, char *uptime)
-{
-	FILE *f;
-	char buf[80];
-
-	f = fopen("/proc/uptime", "r");
-	if (!f) {
-		fprintf(stderr, "Could not open /proc/uptime\n");
-		sprintf(uptime, SE_NOT_VALID);
-		return;
-	}
-
-	if ((fgets(buf, 80, f)) != NULL) {
-		char *value;
-
-		value = strchr(buf, ' ');
-		*value = '\0';
-		sprintf(uptime, "%s", buf);
-	} else {
-		sprintf(uptime, SE_NOT_VALID);
-	}
-
-	fclose(f);
-}
-
 int get_nominal_frequency(void)
 {
 	FILE *f;
@@ -403,13 +381,12 @@ void get_cpu_physc(struct sysentry *unused_se, char *buf)
 	delta_purr = get_delta_value("purr");
 
 	se = get_sysentry("tbr");
-	if (se->value[0] != '\0') {
+	if (se->old_value[0] != '\0') {
 		delta_tb = get_delta_value("tbr");
 
 		physc = delta_purr / delta_tb;
 	} else {
 		elapsed = get_delta_value("time");
-		elapsed = elapsed / 1000000.0;
 
 		se = get_sysentry("timebase");
 		timebase = atoi(se->value);
@@ -436,23 +413,9 @@ void get_cpu_app(struct sysentry *unused_se, char *buf)
 {
 	struct sysentry *se;
 	float timebase, app, elapsed_time;
-	long long new_app, old_app, delta_time;
-	char *descr, uptime[32];
+	long long new_app, old_app;
 
-	se = get_sysentry("time");
-	if (se->old_value[0] == '\0') {
-		/* Single report since boot */
-		get_sysdata("uptime", &descr, uptime);
-
-		if (!strcmp(uptime, SE_NOT_VALID)) {
-			sprintf(buf, "-");
-			return;
-		}
-		elapsed_time = atof(uptime);
-	} else {
-		delta_time = get_delta_value("time");
-		elapsed_time = delta_time / 1000000.0;
-	}
+	elapsed_time = get_delta_value("time");
 
 	se = get_sysentry("timebase");
 	timebase = atof(se->value);
@@ -460,7 +423,8 @@ void get_cpu_app(struct sysentry *unused_se, char *buf)
 	se = get_sysentry("pool_idle_time");
 	new_app = strtoll(se->value, NULL, 0);
 	if (se->old_value[0] == '\0') {
-		old_app = 0;
+		se = get_sysentry("boot_pool_idle_time");
+		old_app = strtoll(se->value, NULL, 0);
 	} else {
 		old_app = strtoll(se->old_value, NULL, 0);
 	}
@@ -515,11 +479,17 @@ void get_cpu_idle_purr(struct sysentry *unused_se, char *buf)
 {
 	double delta_tb, delta_purr, delta_idle_purr;
 	double physc, idle;
+	char *descr;
+	char mode[32];
 
 	delta_tb = get_scaled_tb();
 	delta_purr = get_delta_value("purr");
 	delta_idle_purr = get_delta_value("idle_purr");
 
+	get_sysdata("shared_processor_mode", &descr, mode);
+	if (!strcmp(mode, "Dedicated"))
+		get_sysdata("DedDonMode", &descr, mode);
+
 	/*
 	 * Given that these values are read from different
 	 * sources (purr from lparcfg and idle_purr from sysfs),
@@ -528,10 +498,23 @@ void get_cpu_idle_purr(struct sysentry *unused_se, char *buf)
 	 */
 	if (delta_idle_purr > delta_purr)
 		delta_idle_purr = delta_purr;
+        /*
+	 * Round down delta_purr to delta_tb if delta_tb - delta_purr
+	 * error is under -1%.
+	 */
+	if (((delta_tb - delta_purr + delta_idle_purr) / delta_tb * 100) > -1 && ((delta_tb - delta_purr + delta_idle_purr) / delta_tb * 100) < 0)
+		delta_purr = delta_tb;
 
-	physc = (delta_purr - delta_idle_purr) / delta_tb;
-	idle = (delta_purr / delta_tb) - physc;
-	idle *= 100.00;
+	if (!strcmp(mode, "Capped")) {
+		/* For dedicated - capped mode */
+		physc = (delta_purr - delta_idle_purr) / delta_tb;
+		idle = (delta_purr / delta_tb) - physc;
+		idle *= 100.00;
+	} else {
+		/* For shared and dedicated - donate mode */
+		idle = (delta_tb - delta_purr + delta_idle_purr) / delta_tb;
+		idle *= 100.00;
+	}
 
 	sprintf(buf, "%.2f", idle);
 }
@@ -559,14 +542,30 @@ void get_cpu_idle_spurr(struct sysentry *unused_se, char *buf)
 	double delta_tb, delta_spurr, delta_idle_spurr;
 	double physc, idle;
 	double rfreq;
+	char *descr;
+	char mode[32];
 
 	delta_tb = get_scaled_tb();
 	delta_spurr = get_delta_value("spurr");
 	delta_idle_spurr = get_delta_value("idle_spurr");
 
-	physc = (delta_spurr - delta_idle_spurr) / delta_tb;
-	idle = (delta_spurr / delta_tb) - physc;
-	idle *= 100.00;
+	get_sysdata("shared_processor_mode", &descr, mode);
+	if (!strcmp(mode, "Dedicated"))
+		get_sysdata("DedDonMode", &descr, mode);
+
+	if (delta_spurr > delta_tb)
+		delta_spurr = delta_tb;
+
+	if (!strcmp(mode, "Capped")) {
+		/* For dedicated - capped mode */
+		physc = (delta_spurr - delta_idle_spurr) / delta_tb;
+		idle = (delta_spurr / delta_tb) - physc;
+		idle *= 100.00;
+	} else {
+		/* For shared and dedicated - donate mode */
+		idle = (delta_tb - delta_spurr + delta_idle_spurr) / delta_tb;
+		idle *= 100.00;
+	}
 
 	rfreq = round_off_freq();
 	idle += ((idle * rfreq) / 100);
@@ -716,7 +715,7 @@ int parse_proc_stat()
 	return 0;
 }
 
-void get_smt_state(struct sysentry *se, char *buf)
+void get_processor_type(struct sysentry *se, char *buf)
 {
 	char *value = "?";
 
